@@ -1,5 +1,3 @@
-/* $XdotOrg: xserver/xorg/XTrap/xtrapdi.c,v 1.6 2006/02/10 22:00:19 anholt Exp $ */
-/* $XFree86: xc/programs/Xserver/XTrap/xtrapdi.c,v 1.6tsi Exp $ */
 /*****************************************************************************
 Copyright 1987, 1988, 1989, 1990, 1991 by Digital Equipment Corp., Maynard, MA
 X11R6 Changes Copyright (c) 1994 by Robert Chesler of Absol-Puter, Hudson, NH.
@@ -60,7 +58,7 @@ CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 #include <X11/Xos.h>
 #include <X11/X.h>
 #include <X11/Xproto.h>
-#include "input.h"              /* Server DevicePtr definitions */
+#include "inputstr.h"           /* Server DevicePtr definitions */
 #include "misc.h"               /* Server swapping macros */
 #include "dixstruct.h"          /* Server ClientRec definitions */
 #include "resource.h"           /* Used with the MakeAtom call */
@@ -170,7 +168,7 @@ static ClientList cmd_clients;   /* Linked-list of clients using command key */
 /*----------------------------*
  *  Forward Declarations
  *----------------------------*/
-static void _SwapProc (int (**f1 )(), int (**f2 )());
+static void _SwapProc (int (**f1 )(void), int (**f2 )(void));
 static void sXETrapEvent (xETrapDataEvent *from , xETrapDataEvent *to );
 static int add_accelerator_node (ClientPtr client , ClientList *accel );
 static void remove_accelerator_node (ClientPtr client , ClientList *accel );
@@ -279,7 +277,7 @@ Bool XETrapRedirectDevices()
     /* Do we need to redirect the keyboard device? */
     if (XETrapKbdDev == NULL)
     {
-        if ((XETrapKbdDev = LookupKeyboardDevice()) == NULL)
+        if ((XETrapKbdDev = (DevicePtr)inputInfo.keyboard) == NULL)
         {
             retval = False;
         }
@@ -304,7 +302,7 @@ Bool XETrapRedirectDevices()
 #ifndef VECTORED_EVENTS
     if (XETrapPtrDev == NULL)
     {
-        if ((XETrapPtrDev = LookupPointerDevice()) == 0L)
+        if ((XETrapPtrDev = (DevicePtr)inputInfo.pointer) == 0L)
         {
             retval = False;
         }
@@ -488,7 +486,7 @@ int XETrapCreateEnv(ClientPtr client)
     XETrapEnv *penv = NULL;
     int status = Success;
 
-    if (client->index > MAXCLIENTS)
+    if (client->index >= MAXCLIENTS)
     {
         status = BadImplementation;
     }
@@ -886,14 +884,14 @@ int XETrapConfig(xXTrapConfigReq *request, ClientPtr client)
                 {   /* Client wants the XTrap rtn */
                     if (++(vectored_requests[i]) <= 1L)
                     {   /* first client, so do it */
-                        _SwapProc(&(XETrapProcVector[i]), &(ProcVector[i]));
+                        _SwapProc(&(XETrapProcVector[i]), (int_function *)&(ProcVector[i]));
                     }
                 }
                 else
                 {   /* Client wants the *real* rtn */
                     if (--(vectored_requests[i]) <= 0L)
                     {   /* No more clients using, so do it */
-                        _SwapProc(&(XETrapProcVector[i]), &(ProcVector[i]));
+                        _SwapProc(&(XETrapProcVector[i]), (int_function *)&(ProcVector[i]));
                     }
                 }
                 if (status == Success)
@@ -1094,8 +1092,8 @@ int XETrapRequestVector(ClientPtr client)
             pdata->hdr.client = client->index;  /* stuff client index in hdr */
             if (BitIsTrue(penv->cur.data_config_flags_data,XETrapWinXY))
             {
-                window_ptr = (WindowPtr) LookupDrawable(stuff->id, client);
-                if (window_ptr == 0L)
+		if (Success != dixLookupDrawable(&window_ptr, stuff->id,
+						 client, 0, DixUnknownAccess))
                 {   /* Failed...invalidate the X and Y coordinate data. */
                     pdata->hdr.win_x = -1L;
                     pdata->hdr.win_y = -1L;
@@ -1787,9 +1785,9 @@ static void update_protocol(xXTrapGetReq *reqptr, ClientPtr client)
  * lint from complaining about mixed types. It seems to work, but I would
  * probably classify this as a hack.
  */
-static void _SwapProc( register int (**f1)(), register int (**f2)())
+static void _SwapProc( register int (**f1)(void), register int (**f2)(void))
 {
-    register int (*t1)() = *f1;
+    register int (*t1)(void) = *f1;
     *f1 = *f2;
     *f2 = t1;
 
@@ -2032,7 +2030,7 @@ static void GetSendColorCellsRep(ClientPtr client, xResourceReq *req)
     }
     nmasks = creq->planes;
     length = ((long)npixels + (long)nmasks) * sizeof(Pixel);
-    data = (XETrapDatum *)ALLOCATE_LOCAL(sizeof(XETrapDatum)+length);
+    data = (XETrapDatum *)xalloc(sizeof(XETrapDatum)+length);
     if (!data)
     {
         SendErrorToClient(penv->client, XETrap_avail.data.major_opcode,
@@ -2050,7 +2048,7 @@ static void GetSendColorCellsRep(ClientPtr client, xResourceReq *req)
     {
         SendErrorToClient(penv->client, XETrap_avail.data.major_opcode,
             req->reqType, 0L, retval);
-        DEALLOCATE_LOCAL(data);
+        xfree(data);
         return;
     }
     crep = (xAllocColorCellsReply *)&(data->u.reply);
@@ -2075,7 +2073,7 @@ static void GetSendColorCellsRep(ClientPtr client, xResourceReq *req)
         SendErrorToClient(penv->client, XETrap_avail.data.major_opcode,
             req->reqType, 0L, XETrapErrorBase + BadIO);
     }
-    DEALLOCATE_LOCAL(data);
+    xfree(data);
 }
 static void GetSendColorPlanesRep(ClientPtr client, xResourceReq *req)
 {   /* adapted from ProcAllocColorPlanes() in dispatch.c */
@@ -2103,7 +2101,7 @@ static void GetSendColorPlanesRep(ClientPtr client, xResourceReq *req)
         return;
     }
     length = (long)npixels * sizeof(Pixel);
-    data = (XETrapDatum *)ALLOCATE_LOCAL(sizeof(XETrapDatum)+length);
+    data = (XETrapDatum *)xalloc(sizeof(XETrapDatum)+length);
     if (!data)
     {
         SendErrorToClient(penv->client, XETrap_avail.data.major_opcode,
@@ -2123,7 +2121,7 @@ static void GetSendColorPlanesRep(ClientPtr client, xResourceReq *req)
     {
         SendErrorToClient(penv->client, XETrap_avail.data.major_opcode,
             req->reqType, 0L, retval);
-        DEALLOCATE_LOCAL(data);
+        xfree(data);
         return;
     }
     crep->nPixels = npixels;
@@ -2148,6 +2146,6 @@ static void GetSendColorPlanesRep(ClientPtr client, xResourceReq *req)
         SendErrorToClient(penv->client, XETrap_avail.data.major_opcode,
             req->reqType, 0L, XETrapErrorBase + BadIO);
     }
-    DEALLOCATE_LOCAL(data);
+    xfree(data);
 }
 #endif /* COLOR_REPLIES */
