@@ -29,6 +29,7 @@
 #endif
 #include <mivalidate.h>
 #include <dixstruct.h>
+#include "privates.h"
 #ifdef RANDR
 #include <randrstr.h>
 #endif
@@ -66,8 +67,8 @@ KdDepths    kdDepths[] = {
 
 #define KD_DEFAULT_BUTTONS 5
 
-int                 kdScreenPrivateIndex;
-unsigned long       kdGeneration;
+DevPrivateKey       kdScreenPrivateKey = &kdScreenPrivateKey;
+unsigned long	    kdGeneration;
 
 Bool                kdVideoTest;
 unsigned long       kdVideoTestTime;
@@ -101,7 +102,7 @@ KdSetRootClip (ScreenPtr pScreen, BOOL enable)
     WindowPtr	pChild;
     Bool	WasViewable;
     Bool	anyMarked = FALSE;
-    RegionPtr	pOldClip = 0, bsExposed;
+    RegionPtr	pOldClip = 0;
 #ifdef DO_SAVE_UNDERS
     Bool	dosave = FALSE;
 #endif
@@ -159,12 +160,6 @@ KdSetRootClip (ScreenPtr pScreen, BOOL enable)
     
     if (WasViewable)
     {
-	if (pWin->backStorage)
-	{
-	    pOldClip = REGION_CREATE(pScreen, NullBox, 1);
-	    REGION_COPY(pScreen, pOldClip, &pWin->clipList);
-	}
-
 	if (pWin->firstChild)
 	{
 	    anyMarked |= (*pScreen->MarkOverlappedWindows)(pWin->firstChild,
@@ -188,28 +183,6 @@ KdSetRootClip (ScreenPtr pScreen, BOOL enable)
 	    (*pScreen->ValidateTree)(pWin, NullWindow, VTOther);
     }
 
-    if (pWin->backStorage &&
-	((pWin->backingStore == Always) || WasViewable))
-    {
-	if (!WasViewable)
-	    pOldClip = &pWin->clipList; /* a convenient empty region */
-	bsExposed = (*pScreen->TranslateBackingStore)
-			     (pWin, 0, 0, pOldClip,
-			      pWin->drawable.x, pWin->drawable.y);
-	if (WasViewable)
-	    REGION_DESTROY(pScreen, pOldClip);
-	if (bsExposed)
-	{
-	    RegionPtr	valExposed = NullRegion;
-    
-	    if (pWin->valdata)
-		valExposed = &pWin->valdata->after.exposed;
-	    (*pScreen->WindowExposures) (pWin, valExposed, bsExposed);
-	    if (valExposed)
-		REGION_EMPTY(pScreen, valExposed);
-	    REGION_DESTROY(pScreen, bsExposed);
-	}
-    }
     if (WasViewable)
     {
 	if (anyMarked)
@@ -660,11 +633,6 @@ KdProcessArgument (int argc, char **argv, int i)
 	kdDontZap = TRUE;
 	return 1;
     }
-    if (!strcmp (argv[i], "-nozap"))
-    {
-	kdDontZap = TRUE;
-	return 1;
-    }
     if (!strcmp (argv[i], "-3button"))
     {
 	kdEmulateMiddleButton = FALSE;
@@ -784,10 +752,8 @@ KdAllocatePrivates (ScreenPtr pScreen)
     KdPrivScreenPtr	pScreenPriv;
     
     if (kdGeneration != serverGeneration)
-    {
-	kdScreenPrivateIndex = AllocateScreenPrivateIndex();
-	kdGeneration         = serverGeneration;
-    }
+	kdGeneration = serverGeneration;
+
     pScreenPriv = (KdPrivScreenPtr) xalloc(sizeof (*pScreenPriv));
     if (!pScreenPriv)
 	return FALSE;
@@ -1428,8 +1394,8 @@ KdInitOutput (ScreenInfo    *pScreenInfo,
 }
 
 #ifdef DPMSExtension
-void
-DPMSSet(int level)
+int
+DPMSSet(ClientPtr client, int level)
 {
 }
 
